@@ -2,15 +2,11 @@ package App::Env::Null;
 
 # ABSTRACT: return a snapshot of the current environment
 
+use v5.10;
 use strict;
 use warnings;
 
-our $VERSION = '1.02';
-
-
-
-
-
+our $VERSION = '1.03';
 
 
 
@@ -43,7 +39,7 @@ App::Env::Null - return a snapshot of the current environment
 
 =head1 VERSION
 
-version 1.02
+version 1.03
 
 =head1 SYNOPSIS
 
@@ -57,27 +53,23 @@ This module returns a snapshot of the current environment.  It must
 not be used directly; see B<App::ENV>.  No B<AppOpts> options are
 recognized.
 
-=begin making_pod_coverage_shut_up
-
-=item envs
-
-=end making_pod_coverage_shut_up
+=for Pod::Coverage envs
 
 =head1 SUPPORT
 
 =head2 Bugs
 
-Please report any bugs or feature requests to bug-app-env@rt.cpan.org  or through the web interface at: https://rt.cpan.org/Public/Dist/Display.html?Name=App-Env
+Please report any bugs or feature requests to bug-app-env@rt.cpan.org  or through the web interface at: L<https://rt.cpan.org/Public/Dist/Display.html?Name=App-Env>
 
 =head2 Source
 
 Source is available at
 
-  https://gitlab.com/djerius/App-Env
+  https://gitlab.com/djerius/app-env
 
 and may be cloned from
 
-  https://gitlab.com/djerius/App-Env.git
+  https://gitlab.com/djerius/app-env.git
 
 =head1 SEE ALSO
 
